DECLARE FUNCTION JudgeFinish% (pltPlates() AS INTEGER)
DECLARE SUB MoveHandLR (intHandOrd AS INTEGER, intHandState AS INTEGER, intDir AS INTEGER)
DECLARE SUB MovePlateLR (intHandOrd AS INTEGER, intHandState AS INTEGER, pltPlates() AS INTEGER, intDir AS INTEGER)
DECLARE SUB SwitchHandState (intHandOrd AS INTEGER, intHandState AS INTEGER)
DECLARE SUB InitPlates (intPlates() AS INTEGER)
DECLARE FUNCTION GetShift% ()
DECLARE SUB DrawPlates (intPlates() AS INTEGER)
DECLARE FUNCTION GetPillarPosition% (intZbIdx AS INTEGER)
DECLARE SUB DrawPlate (intPil AS INTEGER, intPlaOrd AS INTEGER, intPlate AS INTEGER)
DECLARE FUNCTION GetYBase% (intZbPlaOrd AS INTEGER)
DECLARE FUNCTION GetPlateRadius% (intPlate AS INTEGER)
DECLARE SUB InitHand (intHandOrd AS INTEGER, intHandState AS INTEGER)
DECLARE SUB DrawHand (intHandOrd AS INTEGER, intHandState AS INTEGER)
DECLARE FUNCTION GetHandYShift% ()
DECLARE FUNCTION GetKey$ ()
DECLARE SUB ClearHand (intHandOrd AS INTEGER, intHandState AS INTEGER)
DECLARE SUB MoveTopPlateLR (intHandOrd AS INTEGER, pltPlates() AS INTEGER, intDir AS INTEGER)
DECLARE FUNCTION CanMove% (intHandOrd AS INTEGER, pltPlates() AS INTEGER, intDir AS INTEGER)
DECLARE FUNCTION GetTopPlate% (intPillar AS INTEGER, pltPlates() AS INTEGER)
DECLARE SUB ClearPlate (intPil AS INTEGER, intPlaOrd AS INTEGER, intPlate AS INTEGER)
DECLARE FUNCTION GetTopPlateOrder% (intPil AS INTEGER, pltPlates() AS INTEGER)

DEFSTR A-Z

CONST True = -1
CONST False = 0

CONST ScreenWidth = 320
CONST ScreenHeight = 200
CONST PlateCount = 7
CONST PlateTopMargin = 5
CONST PlateHeight = 10
CONST PlateMinRadius = 15
CONST PlateMaxRadius = 45
CONST PlateLeftMargin = 10
CONST PillarCount = 3
CONST PillarHeight = 120
CONST HandHeight = 40
CONST HandRadius = 1
CONST HandStateUp = 0
CONST HandStateDown = 1


DIM pltPlates(0 TO PillarCount - 1, 0 TO PlateCount - 1) AS INTEGER
DIM intHandOrd AS INTEGER
DIM intHandState AS INTEGER ' 0 - up, 1 - down
DIM strKey AS STRING

SCREEN 1

CALL InitPlates(pltPlates())
CALL InitHand(intHandOrd, intHandState)
CALL DrawPlates(pltPlates())
CALL DrawHand(intHandOrd, intHandState)
DO
    strKey = GetKey$
    IF strKey = CHR$(27) THEN
        EXIT DO
    END IF
    SELECT CASE strKey
    CASE CHR$(13) ' return
        CALL SwitchHandState(intHandOrd, intHandState)
    CASE CHR$(0) + CHR$(75) ' left
        CALL MovePlateLR(intHandOrd, intHandState, pltPlates(), -1)
    CASE CHR$(0) + CHR$(77) ' right
        CALL MovePlateLR(intHandOrd, intHandState, pltPlates(), 1)
    END SELECT
    IF JudgeFinish%(pltPlates()) THEN
        CLS
        PRINT "You succeeded!"
        EXIT DO
    END IF
LOOP
SYSTEM

FUNCTION CanMove% (intHandOrd AS INTEGER, pltPlates() AS INTEGER, intDir AS INTEGER)

DIM intNewOrd AS INTEGER
DIM intTopPlate1 AS INTEGER
DIM intTopPlate2 AS INTEGER

intNewOrd = (intHandOrd + PillarCount + intDir) MOD PillarCount
intTopPlate1 = GetTopPlate%(intHandOrd, pltPlates())
intTopPlate2 = GetTopPlate%(intNewOrd, pltPlates())
IF intTopPlate1 <> 0 AND (intTopPlate2 = 0 OR intTopPlate2 > intTopPlate1) THEN
    CanMove% = True
ELSE
    CanMove% = False
END IF
    
END FUNCTION

SUB ClearHand (intHandOrd AS INTEGER, intHandState AS INTEGER)

DIM intXPos AS INTEGER
DIM intYTop AS INTEGER
DIM x1 AS INTEGER, x2 AS INTEGER
DIM y1 AS INTEGER, y2 AS INTEGER

intXPos = GetPillarPosition%(intHandOrd)
intYTop = GetHandYShift%
x1 = intXPos - HandRadius: y1 = intYTop
x2 = intXPos + HandRadius: y2 = intYTop + HandHeight
LINE (x1, y1)-(x2, y2), 0, BF

END SUB

SUB ClearPlate (intPil AS INTEGER, intPlaOrd AS INTEGER, intPlate AS INTEGER)

DIM intYBase AS INTEGER
DIM intPilPos AS INTEGER
DIM intRadius AS INTEGER

IF intPlate = 0 THEN
    EXIT SUB
END IF
intPilPos = GetPillarPosition%(intPil)
intYBase = GetYBase%(intPlaOrd)
intRadius = GetPlateRadius%(intPlate)
LINE (intPilPos - intRadius, intYBase)-(intPilPos - 1, intYBase - PlateHeight + 1), 0, BF
LINE (intPilPos + 1, intYBase)-(intPilPos + intRadius, intYBase - PlateHeight + 1), 0, BF

END SUB

SUB DrawHand (intHandOrd AS INTEGER, intHandState AS INTEGER)

DIM intXPos AS INTEGER
DIM intYTop AS INTEGER
DIM x1 AS INTEGER, x2 AS INTEGER
DIM y1 AS INTEGER, y2 AS INTEGER

intXPos = GetPillarPosition%(intHandOrd)
intYTop = GetHandYShift%
x1 = intXPos - HandRadius: y1 = intYTop
x2 = intXPos + HandRadius: y2 = intYTop + HandHeight
IF intHandState = HandStateUp THEN
    LINE (x1, y1)-(x2, y2), , B
ELSE
    LINE (x1, y1)-(x2, y2), , BF
END IF

END SUB

SUB DrawPlate (intPil AS INTEGER, intPlaOrd AS INTEGER, intPlate AS INTEGER)

DIM intYBase AS INTEGER
DIM intPilPos AS INTEGER
DIM intRadius AS INTEGER

IF intPlate = 0 THEN
    EXIT SUB
END IF
intPilPos = GetPillarPosition%(intPil)
intYBase = GetYBase%(intPlaOrd)
intRadius = GetPlateRadius%(intPlate)
LINE (intPilPos - intRadius, intYBase)-(intPilPos + intRadius, intYBase - PlateHeight + 1), , BF

END SUB

SUB DrawPlates (intPlates() AS INTEGER)

DIM i AS INTEGER
DIM j AS INTEGER
DIM intPilPos AS INTEGER ' pilla position

FOR i = 0 TO PillarCount - 1
    intPilPos = GetPillarPosition(i)
    LINE (intPilPos, ScreenHeight - 1)-(intPilPos, ScreenHeight - PillarHeight)
    FOR j = 0 TO PlateCount - 1
        CALL DrawPlate(i, j, intPlates(i, j))
    NEXT
NEXT

END SUB

FUNCTION GetHandYShift%

DIM intYShift AS INTEGER

intYShift = (ScreenHeight - PillarHeight - HandHeight) / 2
GetHandYShift% = intYShift

END FUNCTION

FUNCTION GetKey$

SLEEP
GetKey$ = INKEY$

END FUNCTION

FUNCTION GetPillarPosition% (intZbIdx AS INTEGER)

DIM p AS INTEGER ' position

p = GetShift% + intZbIdx * PlateLeftMargin + (2 * intZbIdx + 1) * PlateMaxRadius
GetPillarPosition% = p

END FUNCTION

FUNCTION GetPlateRadius% (intPlate AS INTEGER)
'intPlate is one-based

DIM intRadius AS INTEGER

intRadius = (intPlate - 1) * (PlateMaxRadius - PlateMinRadius) / (PlateCount - 1) + PlateMinRadius
GetPlateRadius% = intRadius

END FUNCTION

FUNCTION GetShift%

DIM i AS INTEGER
DIM w AS INTEGER
DIM intShift AS INTEGER

w = 0
FOR i = 0 TO PillarCount - 1
    IF w > 0 THEN
        w = w + PlateLeftMargin
    END IF
    w = w + PlateMaxRadius * 2
NEXT
intShift = (ScreenWidth - w) / 2
GetShift% = intShift

END FUNCTION

FUNCTION GetTopPlate% (intPillar AS INTEGER, pltPlates() AS INTEGER)

DIM intTopPlaOrd AS INTEGER
DIM intTopPla AS INTEGER

intTopPla = 0
intTopPlaOrd = GetTopPlateOrder%(intPillar, pltPlates())
IF intTopPlaOrd >= 0 THEN
    intTopPla = pltPlates(intPillar, intTopPlaOrd)
END IF
GetTopPlate% = intTopPla

END FUNCTION

FUNCTION GetTopPlateOrder% (intPil AS INTEGER, pltPlates() AS INTEGER)

DIM i AS INTEGER
DIM intOrd AS INTEGER

intOrd = -1
FOR i = PlateCount - 1 TO 0 STEP -1
    IF pltPlates(intPil, i) <> 0 THEN
        intOrd = i
        EXIT FOR
    END IF
NEXT
GetTopPlateOrder% = intOrd


END FUNCTION

FUNCTION GetYBase% (intZbPlaOrd AS INTEGER)

DIM intYBase AS INTEGER
intYBase = ScreenHeight - intZbPlaOrd * (PlateHeight + PlateTopMargin)
GetYBase% = intYBase

END FUNCTION

SUB InitHand (intHandOrd AS INTEGER, intHandState AS INTEGER)

intHandOrd = 0
intHandState = HandStateUp

END SUB

SUB InitPlates (intPlates() AS INTEGER)

DIM i AS INTEGER
DIM j AS INTEGER

FOR i = 0 TO UBOUND(intPlates, 1)
    IF i = 0 THEN
        FOR j = 0 TO UBOUND(intPlates, 2)
            intPlates(i, j) = PlateCount - j
        NEXT
    END IF
NEXT

END SUB

FUNCTION JudgeFinish% (pltPlates() AS INTEGER)

IF pltPlates(PillarCount - 1, PlateCount - 1) = 1 THEN
    JudgeFinish = True
ELSE
    JudgeFinish = False
END IF

END FUNCTION

SUB MoveHandLR (intHandOrd AS INTEGER, intHandState AS INTEGER, intDir AS INTEGER)

DIM intNewOrd AS INTEGER

intNewOrd = (intHandOrd + PillarCount + intDir) MOD PillarCount
CALL ClearHand(intHandOrd, intHandState)
intHandOrd = intNewOrd
CALL DrawHand(intHandOrd, intHandState)

END SUB

SUB MovePlateLR (intHandOrd AS INTEGER, intHandState AS INTEGER, pltPlates() AS INTEGER, intDir AS INTEGER)

DIM i AS INTEGER
DIM intOldHandOrd AS INTEGER ' used as a backup

intOldHandOrd = intHandOrd
IF intHandState = HandStateUp THEN
    CALL MoveHandLR(intHandOrd, intHandState, intDir)
    EXIT SUB
END IF
IF CanMove%(intHandOrd, pltPlates(), intDir) THEN
    CALL MoveHandLR(intHandOrd, intHandState, intDir)
    CALL MoveTopPlateLR(intOldHandOrd, pltPlates(), intDir)
END IF

END SUB

SUB MoveTopPlateLR (intHandOrd AS INTEGER, pltPlates() AS INTEGER, intDir AS INTEGER)

DIM intNewOrd AS INTEGER
DIM intTopPlaOrd AS INTEGER
DIM intPlate AS INTEGER

intNewOrd = (intHandOrd + PillarCount + intDir) MOD PillarCount
intTopPlaOrd = GetTopPlateOrder%(intHandOrd, pltPlates())
intPlate = pltPlates(intHandOrd, intTopPlaOrd)
CALL ClearPlate(intHandOrd, intTopPlaOrd, intPlate)
pltPlates(intHandOrd, intTopPlaOrd) = 0
intTopPlaOrd = GetTopPlateOrder%(intNewOrd, pltPlates()) + 1
pltPlates(intNewOrd, intTopPlaOrd) = intPlate
CALL DrawPlate(intNewOrd, intTopPlaOrd, intPlate)

END SUB

SUB SwitchHandState (intHandOrd AS INTEGER, intHandState AS INTEGER)

DIM intXPos AS INTEGER
DIM intYTop AS INTEGER
DIM x1 AS INTEGER, x2 AS INTEGER
DIM y1 AS INTEGER, y2 AS INTEGER

IF intHandState = HandStateUp THEN
    intHandState = HandStateDown
ELSE
    intHandState = HandStateUp
END IF
intXPos = GetPillarPosition%(intHandOrd)
intYTop = GetHandYShift%
x1 = intXPos - HandRadius: y1 = intYTop
x2 = intXPos + HandRadius: y2 = intYTop + HandHeight
x1 = x1 + 1: y1 = y1 + 1
x2 = x2 - 1: y2 = y2 - 1
IF intHandState = HandStateUp THEN
    LINE (x1, y1)-(x2, y2), 0, BF
ELSE
    LINE (x1, y1)-(x2, y2), , BF
END IF

END SUB

